"""Socket input models."""

# NOTE:
# - reference API definition for updated query parameter descriptions:
#   https://dev.hume.ai/reference/empathic-voice-interface-evi/chat/chat

from typing import List, Optional

from pydantic import BaseModel


class AudioSettings(BaseModel):
    """Audio settings model."""

    encoding: str = "linear16"  # linear16 is required
    channels: Optional[int] = None
    sample_rate: Optional[int] = None


class Tool(BaseModel):
    """Tool model."""

    description: Optional[str] = None
    fallback_content: Optional[str] = None
    name: str
    parameters: str
    type: str  # "builtin" or "function"


class Context(BaseModel):
    """Context model."""

    text: str
    type: Optional[str]  # editable, persistent, temporary


class BuiltinToolConfig(BaseModel):
    """Built-in Tool Config model."""

    fallback_content: Optional[str] = None
    name: str = "web_search"


class SessionSettings(BaseModel):
    """Session settings model."""

    context: Optional[Context] = None
    custom_session_id: Optional[str] = None
    type: str = "session_settings"
    system_prompt: Optional[str] = None
    audio: Optional[AudioSettings] = None
    language_model_api_key: Optional[str] = None
    builtin_tools: Optional[List[BuiltinToolConfig]] = None
    tools: Optional[List[Tool]]


class TextUserInput(BaseModel):
    """Text user input model. When provided, the text is received as a message by EVI."""

    custom_session_id: Optional[str] = None
    text: str
    type: str = "user_input"


class AssistantInput(BaseModel):
    """Assistant input model. When provided, the input is spoken by EVI."""

    custom_session_id: Optional[str] = None
    text: str
    type: str = "assistant_input"


class PauseAssistantMessage(BaseModel):
    """
    Pause assistant message model.

    When provided, responses from EVI are paused.
    Chat history is still saved and sent after resuming.
    """

    custom_session_id: Optional[str] = None
    type: str = "pause_assistant_message"


class ResumeAssistantMessage(BaseModel):
    """
    Resume assistant message model.

    When provided, responses from EVI are resumed.
    Chat history sent while paused will now be sent.
    """

    custom_session_id: Optional[str] = None
    type: str = "resume_assistant_message"


class ToolErrorMessage(BaseModel):
    """Tool error message model. When provided, the output is a function call error."""

    code: Optional[str] = None
    content: Optional[str] = None
    custom_session_id: Optional[str] = None
    error: str
    level: Optional[str] = "warn"
    tool_call_id: str
    tool_type: str  # "builtin" or "function"
    type: str = "tool_error"


class ToolResponseMessage(BaseModel):
    """Tool response message model. When provided, the output is a function call response."""

    content: str
    custom_session_id: Optional[str] = None
    tool_call_id: str
    tool_name: Optional[str]
    tool_type: str  # "builtin" or "function"
    type: str = "tool_response"



"""Module init."""

from hume._voice.hume_voice_client import HumeVoiceClient
from hume._voice.microphone.microphone_interface import MicrophoneInterface
from hume._voice.models.chats_models import VoiceChat
from hume._voice.models.configs_models import LanguageModelConfig, VoiceConfig, VoiceIdentityConfig
from hume._voice.models.tools_models import VoiceTool
from hume._voice.voice_socket import VoiceSocket

__all__ = [
    "HumeVoiceClient",
    "LanguageModelConfig",
    "MicrophoneInterface",
    "VoiceChat",
    "VoiceConfig",
    "VoiceIdentityConfig",
    "VoiceSocket",
    "VoiceTool",
]



"""Configuration for speech transcription."""

from __future__ import annotations

from dataclasses import dataclass

from hume._common.config_base import ConfigBase


@dataclass
class TranscriptionConfig(ConfigBase["TranscriptionConfig"]):
    """Configuration for speech transcription.

    Args:
        language (str | None): By default, we use an automated language detection method for our
            Speech Prosody, Language, and NER models. However, if you know what language is being spoken
            in your media samples, you can specify it via its BCP-47 tag and potentially obtain more accurate results.
            You can specify any of the following: `zh`, `da`, `nl`, `en`, `en-AU`,
            `en-IN`, `en-NZ`, `en-GB`, `fr`, `fr-CA`, `de`, `hi`, `hi-Latn`, `id`, `it`, `ja`, `ko`, `no`,
            `pl`, `pt`, `pt-BR`, `pt-PT`, `ru`, `es`, `es-419`, `sv`, `ta`, `tr`, or `uk`.
    """

    language: str | None = None



"""Configuration for the facial expression model."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from hume.models import ModelType
from hume.models.config.model_config_base import ModelConfigBase


@dataclass
class FaceConfig(ModelConfigBase):
    """Configuration for the facial expression model.

    Args:
        fps_pred (float | None): Number of frames per second to process. Other frames will be omitted
            from the response.
        prob_threshold (float | None): Face detection probability threshold. Faces detected with a
            probability less than this threshold will be omitted from the response.
        identify_faces (bool | None): Whether to return identifiers for faces across frames.
            If true, unique identifiers will be assigned to face bounding boxes to differentiate different faces.
            If false, all faces will be tagged with an "unknown" ID.
        min_face_size (float | None): Minimum bounding box side length in pixels to treat as a face.
            Faces detected with a bounding box side length in pixels less than this threshold will be
            omitted from the response.
        save_faces (bool | None): Whether to extract and save the detected faces to the artifacts
            directory included in the response.
            This configuration is only available for the batch API.
        descriptions (dict[str, Any] | None): Configuration for Descriptions predictions.
            Descriptions prediction can be enabled by setting "descriptions": {}.
            Currently, Descriptions prediction cannot be further configured with any parameters.
            If missing or null, no descriptions predictions will be generated.
        facs (dict[str, Any] | None): Configuration for FACS predictions.
            FACS prediction can be enabled by setting "facs": {}.
            Currently, FACS prediction cannot be further configured with any parameters.
            If missing or null, no facs predictions will be generated.
    """

    fps_pred: float | None = None
    prob_threshold: float | None = None
    identify_faces: bool | None = None
    min_face_size: float | None = None
    save_faces: bool | None = None
    descriptions: dict[str, Any] | None = None
    facs: dict[str, Any] | None = None

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.FACE



"""Empathic Voice Interface client module."""

import base64
import logging
from importlib.metadata import version
from typing import Any, Optional

import httpx

from hume._voice.mixins.chat_mixin import ChatMixin
from hume._voice.mixins.chats_mixin import ChatsMixin
from hume._voice.mixins.configs_mixin import ConfigsMixin
from hume._voice.mixins.tools_mixin import ToolsMixin

logger = logging.getLogger(__name__)


def generate_client_id(api_key: str, secret_key: str) -> str:
    """Generate a Base64 encoded client ID from API key and secret key.

    Args:
        api_key (str): The API key provided by Hume.
        secret_key (str): The secret key provided by Hume.

    Returns:
        str: Base64 encoded client ID.
    """
    auth_string = f"{api_key}:{secret_key}"
    return base64.b64encode(auth_string.encode()).decode()


def fetch_access_token(client_id: str, host: str = "api.hume.ai", timeout: int = 5) -> str:
    """Fetch an access token using the client ID.

    Args:
        client_id (str): The Base64 encoded client ID.
        host (str): The Hume API host. Defaults to "api.hume.ai".
        timeout (int): Timeout for the HTTP request. Defaults to 5 seconds.

    Returns:
        str: Access token.

    Raises:
        ValueError: If the access token is not found in the response.
    """
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
        "Authorization": f"Basic {client_id}",
    }
    data = {
        "grant_type": "client_credentials",
    }
    with httpx.Client(timeout=timeout) as client:
        response = client.post(f"https://{host}/oauth2-cc/token", headers=headers, data=data)
        response_data = response.json()
    if "access_token" not in response_data:
        raise ValueError("Access token not found in response")
    return response_data["access_token"]


class HumeVoiceClient(ChatMixin, ChatsMixin, ConfigsMixin, ToolsMixin):
    """Empathic Voice Interface client.

    This client integrates with the Hume API to provide various EVI-related operations,
    including chat and configuration management.

    Attributes:
        _token (Optional[str]): The access token for authentication.
        enable_audio (bool): Flag indicating whether audio playback is enabled.
    """

    def __init__(self, api_key: str, secret_key: Optional[str] = None, **kwargs: Any):
        """
        Initialize the HumeVoiceClient.

        Args:
            api_key (str): The API key provided by Hume.
            secret_key (Optional[str]): The secret key provided by Hume. Required for token-based authentication.
            **kwargs: Additional arguments to pass to the parent class initializer.
        """
        super().__init__(api_key, **kwargs)
        self._token: Optional[str] = None
        if secret_key:
            client_id = generate_client_id(api_key, secret_key)
            self._token = fetch_access_token(client_id)

    def _get_client_headers(self) -> dict[str, str]:
        """
        Get the headers required for API requests.

        Returns:
            dict[str, str]: A dictionary of headers.
        """
        headers = {
            "X-Hume-Client-Name": "python_sdk",
            "X-Hume-Client-Version": version("hume"),
        }
        if self._token:
            headers["Authorization"] = f"Bearer {self._token}"
        else:
            headers["X-Hume-Api-Key"] = self._api_key
        return headers



"""Client operations for managing EVI configurations."""

from __future__ import annotations

import logging
from typing import Iterator

from hume._common.client_base import ClientBase
from hume._common.utilities.paging_utilities import Paging
from hume._voice.models.configs_models import (
    ConfigResponse,
    ConfigsResponse,
    LanguageModelConfig,
    PostConfigRequest,
    PostPromptRequest,
    PromptMeta,
    PromptResponse,
    VoiceConfig,
    VoiceIdentityConfig,
)
from hume._voice.models.tools_models import ToolMeta, VoiceTool
from hume.error.hume_client_exception import HumeClientException

logger = logging.getLogger(__name__)


# pylint: disable=redefined-builtin
class ConfigsMixin(ClientBase):
    """Client operations for managing EVI configurations."""

    DEFAULT_VOICE_NAME = "ITO"

    def create_config(
        self,
        *,
        name: str,
        prompt: str,
        description: str | None = None,
        voice_identity_config: VoiceIdentityConfig | None = None,
        tools: list[VoiceTool] | None = None,
        language_model: LanguageModelConfig | None = None,
    ) -> VoiceConfig:
        """Create a new EVI config.

        Args:
            name (str): Config name.
            prompt (str): System prompt text.
            description (str | None): Config description.
        """
        post_prompt_request = PostPromptRequest(name=name, version_description=description, text=prompt)
        post_prompt_body = post_prompt_request.to_json_str()
        endpoint = self._build_endpoint("evi", "prompts")
        response = self._request(endpoint, method="POST", body_json_str=post_prompt_body)
        prompt_response = PromptResponse.model_validate_json(response.text)
        prompt_meta = PromptMeta(id=prompt_response.id, version=prompt_response.version)

        tool_metas = None if tools is None else [ToolMeta(id=tool.id, version=0) for tool in tools]
        post_config_request = PostConfigRequest(
            name=name,
            version_description=description,
            prompt=prompt_meta,
            voice=voice_identity_config,
            tools=tool_metas,
            language_model=language_model,
        )
        post_config_body = post_config_request.to_json_str()
        endpoint = self._build_endpoint("evi", "configs")
        response = self._request(endpoint, method="POST", body_json_str=post_config_body)
        config_response = ConfigResponse.model_validate_json(response.text)

        return self._config_from_response(config_response)

    def get_config(self, id: str, _version: int | None = None) -> VoiceConfig:
        """Get an EVI config by its ID.

        Args:
            id (str): Config ID.
        """
        route = f"configs/{id}" if _version is None else f"configs/{id}/version/{_version}"
        endpoint = self._build_endpoint("evi", route)
        response = self._request(endpoint, method="GET")
        configs_response = ConfigsResponse.model_validate_json(response.text)
        if len(configs_response.configs_page) == 0:
            raise HumeClientException(f"Config not found with ID: {id}")

        return self._config_from_response(configs_response.configs_page[0])

    def _iter_config_versions(self, id: str) -> Iterator[VoiceConfig]:
        endpoint = self._build_endpoint("evi", f"configs/{id}")
        for page_number in range(self.PAGING_LIMIT):
            paging = Paging(page_size=self._page_size, page_number=page_number)
            response = self._request(endpoint, method="GET", paging=paging)
            configs_response = ConfigsResponse.model_validate_json(response.text)
            if len(configs_response.configs_page) == 0:
                break
            for res in configs_response.configs_page:
                yield self._config_from_response(res)

    def _config_from_response(self, config_response: ConfigResponse) -> VoiceConfig:
        prompt_response = config_response.prompt
        prompt = prompt_response.text if prompt_response is not None else None

        return VoiceConfig(
            id=config_response.id,
            name=config_response.name,
            description=config_response.version_description,
            created_on=config_response.created_on,
            modified_on=config_response.modified_on,
            prompt=prompt,
            voice=config_response.voice,
        )

    def iter_configs(self) -> Iterator[VoiceConfig]:
        """Iterate over existing EVI configs."""
        endpoint = self._build_endpoint("evi", "configs")
        for page_number in range(self.PAGING_LIMIT):
            paging = Paging(page_size=self._page_size, page_number=page_number)
            response = self._request(endpoint, method="GET", paging=paging)
            configs_response = ConfigsResponse.model_validate_json(response.text)
            if len(configs_response.configs_page) == 0:
                break
            for res in configs_response.configs_page:
                yield self._config_from_response(res)

    def delete_config(self, id: str, _version: int | None = None) -> None:
        """Delete an EVI config.

        Args:
            id (str): Config ID.
        """
        route = f"configs/{id}" if _version is None else f"configs/{id}/version/{_version}"
        endpoint = self._build_endpoint("evi", route)
        self._request(endpoint, method="DELETE")



import asyncio
import base64
import json
import logging
import urllib.parse
from contextlib import asynccontextmanager
from typing import Any, AsyncIterator, Awaitable, Callable, ClassVar, Dict, Optional, Union

import websockets
import websockets.client

from hume._common.client_base import ClientBase
from hume._common.protocol import Protocol
from hume._voice.asyncio_utilities import Stream
from hume._voice.audio_player import AudioPlayer
from hume._voice.voice_socket import VoiceSocket
from hume.error.hume_client_exception import HumeClientException

logger = logging.getLogger(__name__)

OpenCloseHandlerType = Union[Callable[[], None], Callable[[], Awaitable[None]]]
MessageHandlerType = Union[Callable[[dict], None], Callable[[dict], Awaitable[None]]]
ErrorHandlerType = Union[Callable[[Exception], None], Callable[[Exception], Awaitable[None]]]


class ChatMixin(ClientBase):
    """Client operations for EVI WebSocket connections.

    This mixin provides methods to handle WebSocket connections, messages, and errors for the
    Empathic Voice Interface (EVI) API.
    """

    DEFAULT_MAX_PAYLOAD_SIZE_BYTES: ClassVar[int] = 2**24

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """
        Initialize the ChatMixin.

        Args:
            *args: Additional positional arguments for the parent class.
            **kwargs: Additional keyword arguments for the parent class.
        """
        super().__init__(*args, **kwargs)
        self.audio_player: Optional[AudioPlayer] = None
        self._audio_task: Optional[asyncio.Task[None]] = None

    @asynccontextmanager
    async def connect(
        self,
        config_id: Optional[str] = None,
        resumed_chat_group_id: Optional[str] = None,
    ) -> AsyncIterator[VoiceSocket]:
        """Connect to the EVI API.

        Args:
            config_id (Optional[str]): Config ID.
            chat_group_id (Optional[str]): Chat group ID.
        """
        uri = self._build_uri(config_id, resumed_chat_group_id)
        logger.info("Connecting to EVI API at %s", uri)

        try:
            async with websockets.connect(
                uri,
                extra_headers=self._get_client_headers(),
                close_timeout=self._close_timeout,
                open_timeout=self._open_timeout,
                max_size=self.DEFAULT_MAX_PAYLOAD_SIZE_BYTES,
            ) as protocol:
                yield VoiceSocket(protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            if exc.status_code == 401:  # Unauthorized
                raise HumeClientException("HumeVoiceClient initialized with invalid API key.") from exc
            raise HumeClientException("Unexpected error when creating EVI API connection") from exc

    async def _handle_messages(
        self,
        voice_socket: VoiceSocket,
        byte_strs: Stream,
        on_message: Optional[MessageHandlerType],
        on_error: Optional[ErrorHandlerType],
        enable_audio: bool,
    ) -> None:
        """Handle incoming messages from the WebSocket.

        Args:
            voice_socket (VoiceSocket): The WebSocket connection.
            byte_strs (Stream): Stream to handle audio bytes.
            on_message (Optional[MessageHandlerType]): Handler for incoming messages.
            on_error (Optional[ErrorHandlerType]): Handler for errors.
            enable_audio (bool): Flag indicating whether audio playback is enabled.
        """
        try:
            async for socket_message in voice_socket:
                message = json.loads(socket_message)
                await self._process_message(message, byte_strs, on_message, enable_audio)
        except Exception as exc:
            await self._handle_error(exc, on_error)
            raise

    async def _process_message(
        self, message: dict, byte_strs: Stream, on_message: Optional[MessageHandlerType], enable_audio: bool
    ) -> None:
        """Process an individual message from the WebSocket.

        Args:
            message (dict): The message received from the WebSocket.
            byte_strs (Stream): Stream to handle audio bytes.
            on_message (Optional[MessageHandlerType]): Handler for incoming messages.
            enable_audio (bool): Flag indicating whether audio playback is enabled.
        """
        if on_message is not None:
            if asyncio.iscoroutinefunction(on_message):
                await on_message(message)
            else:
                on_message(message)
        if enable_audio:
            if message["type"] == "user_interruption":
                logger.debug("Received user_interruption message")
                if self.audio_player:
                    self.audio_player.stop_audio()
            elif message["type"] == "audio_output":
                message_str: str = message["data"]
                message_bytes = base64.b64decode(message_str.encode("utf-8"))
                await byte_strs.put(message_bytes)

    async def _audio_playback(self, byte_strs: Stream, enable_audio: bool) -> None:
        """Play audio from the stream of bytes.

        Args:
            byte_strs (Stream): Stream to handle audio bytes.
            enable_audio (bool): Flag indicating whether audio playback is enabled.
        """
        if enable_audio and self.audio_player:
            async for byte_str in byte_strs:
                await self.audio_player.play_audio(byte_str)

    async def _handle_error(self, exc: Exception, on_error: Optional[ErrorHandlerType]) -> None:
        """Handle an error that occurs during WebSocket communication.

        Args:
            exc (Exception): The exception that occurred.
            on_error (Optional[ErrorHandlerType]): Handler for errors.
        """
        if on_error is not None:
            if asyncio.iscoroutinefunction(on_error):
                await on_error(exc)
            else:
                on_error(exc)

    async def _handle_open_close(self, handler: Optional[OpenCloseHandlerType]) -> None:
        """Handle the opening or closing of the WebSocket connection.

        Args:
            handler (Optional[OpenCloseHandlerType]): Handler for open/close events.
        """
        if handler is not None:
            if asyncio.iscoroutinefunction(handler):
                await handler()
            else:
                handler()

    @asynccontextmanager
    async def connect_with_handlers(
        self,
        config_id: Optional[str] = None,
        resumed_chat_group_id: Optional[str] = None,
        on_open: Optional[OpenCloseHandlerType] = None,
        on_close: Optional[OpenCloseHandlerType] = None,
        on_error: Optional[ErrorHandlerType] = None,
        on_message: Optional[MessageHandlerType] = None,
        enable_audio: bool = True,
    ) -> AsyncIterator[VoiceSocket]:
        """Connect to the EVI API with socket handlers and optional audio playback.

        Args:
            config_id (Optional[str]): Config ID.
            chat_group_id (Optional[str]): Chat group ID.
            on_open (Optional[OpenCloseHandlerType]): Handler for when the connection is opened.
            on_message (Optional[MessageHandlerType]): Handler for when a message is received.
            on_error (Optional[ErrorHandlerType]): Handler for when an error occurs.
            on_close (Optional[OpenCloseHandlerType]): Handler for when the connection is closed.
            enable_audio (bool): Flag indicating whether audio playback is enabled. Defaults to True.

        Yields:
            AsyncIterator[VoiceSocket]: The WebSocket connection.
        """
        uri = self._build_uri(config_id, resumed_chat_group_id)
        logger.info("Connecting to EVI API at %s", uri)

        self.audio_player = AudioPlayer() if enable_audio else None

        try:
            async with websockets.connect(
                uri,
                extra_headers=self._get_client_headers(),
                close_timeout=self._close_timeout,
                open_timeout=self._open_timeout,
                max_size=self.DEFAULT_MAX_PAYLOAD_SIZE_BYTES,
            ) as protocol:
                await self._handle_open_close(on_open)

                voice_socket = VoiceSocket(protocol)
                byte_strs: Stream = Stream.new()

                recv_task = asyncio.create_task(
                    self._handle_messages(voice_socket, byte_strs, on_message, on_error, enable_audio)
                )
                if enable_audio:
                    self._audio_task = asyncio.create_task(self._audio_playback(byte_strs, enable_audio))

                yield voice_socket

                if enable_audio and self._audio_task:
                    await asyncio.gather(recv_task, self._audio_task)
                else:
                    await recv_task

        except websockets.exceptions.InvalidStatusCode as exc:
            await self._handle_error(exc, on_error)
            if exc.status_code == 401:  # Unauthorized
                raise HumeClientException("HumeVoiceClient initialized with invalid API key.") from exc
            raise HumeClientException("Unexpected error when creating EVI API connection") from exc
        except Exception as exc:
            await self._handle_error(exc, on_error)
            raise
        finally:
            if self._audio_task is not None:
                self._audio_task.cancel()
                try:
                    await self._audio_task
                except asyncio.CancelledError:
                    pass
            await self._handle_open_close(on_close)

    def _build_uri(self, config_id: Optional[str], chat_group_id: Optional[str]) -> str:
        """Build the URI for connecting to the EVI API.

        Args:
            config_id (Optional[str]): Config ID.
            chat_group_id (Optional[str]): Chat group ID.

        Returns:
            str: The constructed URI.

        Raises:
            HumeClientException: If both config_id and chat_group_id are provided.
        """
        uri_base = self._build_endpoint("evi", "chat", Protocol.WS)
        if config_id and chat_group_id:
            raise HumeClientException(
                "If resuming from a chat_group_id you must not provide a config_id. "
                "The original config for the chat group will be used automatically."
            )
        params: Dict[str, Any] = {}
        if config_id:
            params["config_id"] = config_id
        if chat_group_id:
            params["resumed_chat_group_id"] = chat_group_id
        encoded_params = urllib.parse.urlencode(params)
        return f"{uri_base}?{encoded_params}"



from __future__ import annotations

from hume._common.utilities.model_utilities import BaseModel


class Paging(BaseModel):
    """HTTP response paging parameters."""

    page_size: int | None
    page_number: int = 0



"""Hume API client exception."""


class HumeClientException(Exception):
    """Hume API client exception."""

    @classmethod
    def from_error(cls, error_code: str, error_message: str) -> "HumeClientException":
        """Create an exception from an error.

        Args:
            error_code (str): Error code associated with the given error.
            error_message (str): Error message.

        Returns:
            HumeClientException: Exception object with the formatted error message.
        """
        return cls(f"hume({error_code}): {error_message}")



"""Module init."""

from hume._measurement.batch.batch_job import BatchJob
from hume._measurement.batch.batch_job_details import BatchJobDetails
from hume._measurement.batch.batch_job_state import BatchJobState
from hume._measurement.batch.batch_job_status import BatchJobStatus
from hume._measurement.batch.hume_batch_client import HumeBatchClient
from hume._measurement.batch.transcription_config import TranscriptionConfig

__all__ = [
    "BatchJob",
    "BatchJobDetails",
    "BatchJobState",
    "BatchJobStatus",
    "HumeBatchClient",
    "TranscriptionConfig",
]



"""Utilities for audio playback.

This module provides the AudioPlayer class for playing audio byte streams using the system's audio output device.
"""

import asyncio
from io import BytesIO
from typing import Optional

import simpleaudio as sa
from pydub import AudioSegment

# NOTE:
# - expects that byte_str is a valid audio file with the appropriate headers
# - explicitly uses simpleaudio for audio playback


class AudioPlayer:
    """Audio player for playing audio byte streams using the system's audio output device.

    This class uses simpleaudio for audio playback and pydub for handling audio data.
    """

    def __init__(self) -> None:
        """Initialize the AudioPlayer."""
        self._playback_object: Optional[sa.PlayObject] = None
        self._stop_event = asyncio.Event()
        self._play_lock = asyncio.Lock()

    async def play_audio(self, byte_str: bytes) -> None:
        """Play a byte string of audio data with the system audio output device.

        Args:
            byte_str (bytes): Byte string of audio data.
        """
        async with self._play_lock:
            # Wait for any existing playback to complete
            if self._playback_object is not None:
                self._playback_object.wait_done()

            # Reset the stop event for new playback
            self._stop_event.clear()

            # Convert byte string to an AudioSegment
            segment = AudioSegment.from_file(BytesIO(byte_str))
            audio_data = segment.raw_data
            num_channels = segment.channels
            bytes_per_sample = segment.sample_width
            sample_rate = segment.frame_rate

            def _play_audio_segment() -> None:
                """Play the audio segment using simpleaudio."""
                wave_obj = sa.WaveObject(audio_data, num_channels, bytes_per_sample, sample_rate)
                self._playback_object = wave_obj.play()

            # Offload the playback to a separate thread
            await asyncio.to_thread(_play_audio_segment)

    def stop_audio(self) -> None:
        """Stop the current audio playback."""
        # Signal to stop playback
        self._stop_event.set()

        if self._playback_object:
            self._playback_object.stop()
            self._playback_object = None



"""Batch job state."""

from __future__ import annotations

from dataclasses import dataclass

from hume._measurement.batch.batch_job_status import BatchJobStatus


@dataclass
class BatchJobState:
    """Batch job state.

    Args:
        status (BatchJobStatus): Status of the batch job.
        created_timestamp_ms (int | None): Time when job was created.
        started_timestamp_ms (int | None): Time when job started.
        ended_timestamp_ms (int | None): Time when job ended.
    """

    status: BatchJobStatus
    created_timestamp_ms: int | None
    started_timestamp_ms: int | None
    ended_timestamp_ms: int | None



"""Configuration for the named-entity emotion model."""

from __future__ import annotations

from dataclasses import dataclass

from hume.models import ModelType
from hume.models.config.model_config_base import ModelConfigBase


@dataclass
class NerConfig(ModelConfigBase):
    """Configuration for the named-entity emotion model.

    This model is only available for the batch API.

    Args:
        identify_speakers (bool | None): Whether to return identifiers for speakers over time. If true,
            unique identifiers will be assigned to spoken words to differentiate different speakers. If false,
            all speakers will be tagged with an "unknown" ID.
            This configuration is only available for the batch API.
    """

    identify_speakers: bool | None = None

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.NER



"""Voice socket connection."""

from __future__ import annotations

import json
import logging
from pathlib import Path
from typing import Any, AsyncIterator, ClassVar, List, Optional, Union

from pydub import AudioSegment
from websockets.client import WebSocketClientProtocol as WebSocket

from hume._common.utilities.typing_utilities import JsonObject
from hume._voice.socket_inputs import (
    AssistantInput,
    AudioSettings,
    BuiltinToolConfig,
    Context,
    PauseAssistantMessage,
    ResumeAssistantMessage,
    SessionSettings,
    TextUserInput,
    Tool,
    ToolErrorMessage,
    ToolResponseMessage,
)

logger = logging.getLogger(__name__)


class VoiceSocket:
    """Voice socket connection."""

    DEFAULT_CUT_MS: ClassVar[int] = 250
    DEFAULT_NUM_CHANNELS: ClassVar[int] = 1
    DEFAULT_SAMPLE_RATE: ClassVar[int] = 44_100

    def __init__(self, protocol: WebSocket):
        """Construct a `VoiceSocket`.

        Args:
            protocol (WebSocketClientProtocol): Protocol instance from websockets library.

        Raises:
            HumeClientException: If there is an error processing media over the socket connection.
        """
        self._protocol = protocol

        self._num_channels = self.DEFAULT_NUM_CHANNELS
        self._sample_rate = self.DEFAULT_SAMPLE_RATE

    async def __aiter__(self) -> AsyncIterator[Any]:
        """Async iterator for the voice socket."""
        async for message in self._protocol:
            yield message

    async def send(self, byte_str: bytes) -> None:
        """Send a byte string over the voice socket.

        Args:
            byte_str (bytes): Byte string to send.
        """
        await self._protocol.send(byte_str)

    async def send_json(self, message: JsonObject) -> None:
        """Send JSON as a byte string over the voice socket.

        Args:
            message (JsonObject): A dictionary representing a full JSON payload to the server.
        """
        await self._protocol.send(json.dumps(message).encode("utf-8"))

    async def recv(self) -> Any:
        """Receive a message on the voice socket."""
        await self._protocol.recv()

    async def update_session_settings(
        self,
        *,
        sample_rate: Optional[int] = None,
        num_channels: Optional[int] = None,
        custom_session_id: Optional[str] = None,
        system_prompt: Optional[str] = None,
        language_model_api_key: Optional[str] = None,
        builtin_tools: Optional[List[BuiltinToolConfig]] = None,
        tools: Optional[List[Tool]] = None,
        context_text: Optional[str] = None,
        context_type: Optional[str] = None,
    ) -> None:
        """
        Update the EVI session settings.

        This method allows updating various settings of the EVI session such as
        audio configurations, session ID, system prompt, API key for the language model,
        and tools configurations. If the number of channels or sample rate is specified,
        it updates the instance variables `_num_channels` and `_sample_rate` accordingly.

        Args:
            sample_rate (Optional[int]): The sample rate for audio data; updates session state variable if provided.
            num_channels (Optional[int]): The number of audio channels; updates session state variable if provided.
            custom_session_id (Optional[str]): A custom session ID to manage conversational state.
            context_text (Optional[str]): User context to inject.
            context_type (Optional[str]): The persistence level of the injected context. Allowed values include
            "editable", "persistent", and "temporary".
            system_prompt (Optional[str]): Instructions for how the system should respond to the user.
            Set to null to use the default system prompt.
            language_model_api_key (Optional[str]): Third party API key for the language model used for non-Hume models.
            builtin_tools (Optional[List[BuiltinToolConfig]]): List of built-in tool configurations session use.
            tools (Optional[List[Tool]]): List of custom tools configurations to be used in the session.

        Raises:
            HumeClientException: If there is an error processing media over the socket connection.
        """
        if num_channels is not None:
            self._num_channels = num_channels
        if sample_rate is not None:
            self._sample_rate = sample_rate

        context = None
        if context_text and context_type:
            context = Context(text=context_text, type=context_type)

        session_settings = SessionSettings(
            custom_session_id=custom_session_id,
            type="session_settings",
            system_prompt=system_prompt,
            audio=AudioSettings(
                channels=num_channels,
                sample_rate=sample_rate,
            ),
            context=context,
            language_model_api_key=language_model_api_key,
            builtin_tools=builtin_tools,
            tools=tools,
        )

        settings_dict = session_settings.model_dump(exclude_none=True)

        logger.info(f"Updating session settings to: {settings_dict}")
        message = json.dumps(settings_dict)
        await self._protocol.send(message)

    async def send_file(self, filepath: Union[str, Path]) -> None:
        """Send a file over the voice socket.

        Args:
            filepath (str | Path): Filepath to the file to send over the socket.
        """
        # Create a Path object from the filepath string
        path_object = Path(filepath)

        with path_object.open("rb") as f:
            segment: AudioSegment = AudioSegment.from_file(f)
            segment = segment.set_frame_rate(self._sample_rate).set_channels(self._num_channels)
            audio_bytes = segment.raw_data
            await self._protocol.send(audio_bytes)

    async def pause_assistant(self, custom_session_id: str | None = None) -> None:
        """Pause assistant.

        Args:
            custom_session_id (str, optional): Session ID for managing conversational state.
        """
        pause_assistant_message = PauseAssistantMessage(custom_session_id=custom_session_id)
        pause_assistant_message_dict = pause_assistant_message.model_dump(exclude_none=True)
        message = json.dumps(pause_assistant_message_dict)

        await self._protocol.send(message)

    async def resume_assistant(self, custom_session_id: str | None = None) -> None:
        """Resume assistant.

        Args:
            custom_session_id (str, optional): Session ID for managing conversational state.
        """
        resume_assistant_message = ResumeAssistantMessage(custom_session_id=custom_session_id)
        resume_assistant_message_dict = resume_assistant_message.model_dump(exclude_none=True)
        message = json.dumps(resume_assistant_message_dict)

        await self._protocol.send(message)

    async def send_assistant_input(self, input_text: str, custom_session_id: str | None = None) -> None:
        """Send assistant input.

        Args:
            text (str): Text to be synthesized.
            custom_session_id (str, optional): Session ID for managing conversational state.
        """
        assistant_input = AssistantInput(text=input_text, custom_session_id=custom_session_id)
        assistant_input_dict = assistant_input.model_dump(exclude_none=True)
        message = json.dumps(assistant_input_dict)

        await self._protocol.send(message)

    async def send_text_input(self, input_text: str, custom_session_id: str | None = None) -> None:
        """Send text input.

        Args:
            text (str): Text input to be sent.
            custom_session_id (str, optional): Session ID for managing conversational state.
        """
        text_user_input = TextUserInput(text=input_text, custom_session_id=custom_session_id)
        text_user_input_dict = text_user_input.model_dump(exclude_none=True)
        message = json.dumps(text_user_input_dict)

        await self._protocol.send(message)

    async def send_tool_response(
        self,
        content: str,
        tool_call_id: str,
        tool_type: str,
        custom_session_id: Optional[str] = None,
        tool_name: Optional[str] = None,
    ) -> None:
        """Send tool response.

        Args:
            content (str): Content of the tool response.
            tool_call_id (str): ID of the tool call.
            tool_type (str): Type of the tool, either 'builtin' or 'function'.
            custom_session_id (str, optional): Session ID for managing conversational state.
            tool_name (str, optional): Name of the tool.
        """
        tool_response = ToolResponseMessage(
            content=content,
            tool_call_id=tool_call_id,
            tool_type=tool_type,
            custom_session_id=custom_session_id,
            tool_name=tool_name,
        )
        tool_response_dict = tool_response.model_dump(exclude_none=True)
        message = json.dumps(tool_response_dict)

        await self._protocol.send(message)

    async def send_tool_error(
        self,
        error: str,
        tool_call_id: str,
        tool_type: str,
        custom_session_id: Optional[str] = None,
        code: Optional[str] = None,
        content: Optional[str] = None,
        level: Optional[str] = "warn",
    ) -> None:
        """Send tool error.

        Args:
            error (str): Error message.
            tool_call_id (str): ID of the tool call.
            tool_type (str): Type of the tool, either 'builtin' or 'function'.
            custom_session_id (str, optional): Session ID for managing conversational state.
            code (str, optional): Error code.
            content (str, optional): Content of the error message.
            level (str, optional): Error level.
        """
        tool_error = ToolErrorMessage(
            error=error,
            tool_call_id=tool_call_id,
            tool_type=tool_type,
            custom_session_id=custom_session_id,
            code=code,
            content=content,
            level=level,
        )
        tool_error_dict = tool_error.model_dump(exclude_none=True)
        message = json.dumps(tool_error_dict)

        await self._protocol.send(message)

    async def close(self) -> None:
        """Close the underlying socket."""
        await self._protocol.close()



"""Configuration for the language emotion model."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from hume.models import ModelType
from hume.models.config.model_config_base import ModelConfigBase


@dataclass
class LanguageConfig(ModelConfigBase):
    """Configuration for the language emotion model.

    Args:
        granularity (str | None): The granularity at which to generate predictions.
            Accepted values are `word`, `sentence`, `utterance`, or `conversational_turn`.
            The default is `utterance`.
            `utterance` corresponds to a natural pause or break in conversation
            `conversational_turn` corresponds to a change in speaker.
            This configuration is available for the streaming API, but only with values `word` and `sentence`.
        identify_speakers (bool | None): Whether to return identifiers for speakers over time.
            If true, unique identifiers will be assigned to spoken words to differentiate different speakers.
            If false, all speakers will be tagged with an "unknown" ID.
            This configuration is only available for the batch API.
        sentiment (dict[str, Any] | None): Configuration for Sentiment predictions.
            Sentiment prediction can be enabled by setting "sentiment": {}.
            Currently, Sentiment prediction cannot be further configured with any parameters.
            If missing or null, no sentiment predictions will be generated.
        toxicity (dict[str, Any] | None): Configuration for Toxicity predictions.
            Toxicity prediction can be enabled by setting "toxicity": {}.
            Currently, Toxicity prediction cannot be further configured with any parameters.
            If missing or null, no toxicity predictions will be generated.
    """

    granularity: str | None = None
    identify_speakers: bool | None = None
    sentiment: dict[str, Any] | None = None
    toxicity: dict[str, Any] | None = None

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.LANGUAGE
